/*
 * Copyright (C) 2012, Eric Wong <normalperson@yhbt.net>
 * License: GPLv3 or later (see COPYING for details)
 *
 * TODO: ensure compliance with relevant RFCs.
 *       This is only enough to work with MogileFS.
 */
#include "cmogstored.h"
#include "http.h"

void mog_http_delete(struct mog_http *http, char *buf)
{
	int rc;
	char *path = mog_http_path(http, buf);

	if (!path) goto forbidden; /* path traversal attack */
	assert(path[0] == '/' && "bad path");
	if (path[1] == '\0') goto forbidden;

	rc = mog_unlink(http->svc, path);
	if (rc == 0) {
		mog_http_resp(http, "204 No Content", true);
		return;
	}

	switch (errno) {
	case EPERM:
	case EISDIR:
	case EACCES:
forbidden:
		mog_http_resp(http, "403 Forbidden", true);
		return;
	case ENOENT:
		mog_http_resp(http, "404 Not Found", true);
		return;
	}
	PRESERVE_ERRNO(do {
		mog_http_resp(http, "500 Internal Server Error", true);
	} while(0));
}

void mog_http_mkcol(struct mog_http *http, char *buf)
{
	int rc;
	struct stat sb;
	char *path = mog_http_path(http, buf);
	const char *r;

	if (!path) goto forbidden; /* path traversal attack */

	rc = mog_mkdir(http->svc, path, http->svc->mkcol_perms);
	if (rc < 0 && (errno == EEXIST)) {
		/* directory may exist already */
		rc = mog_stat(http->svc, path, &sb);
		if (rc < 0) goto err;
		assert(rc == 0 && "mog_stat() returned > 0");

		r = S_ISDIR(sb.st_mode) ? "204 No Content" : "409 Conflict";
		mog_http_resp(http, r, true);
		return;
	}
	if (rc == 0) {
		mog_http_resp(http, "201 Created", true);
		return;
	}

err:
	switch (errno) {
	case EACCES:
forbidden:
		mog_http_resp(http, "403 Forbidden", true);
		return;
	}
	PRESERVE_ERRNO(do {
		mog_http_resp(http, "500 Internal Server Error", true);
	} while(0));
}
