
#line 1 "mgmt_parser.rl"
/*
 * Copyright (C) 2012, Eric Wong <normalperson@yhbt.net>
 * License: GPLv3 or later (see COPYING for details)
 */
#include "cmogstored.h"
#include "mgmt.h"

#line 49 "mgmt_parser.rl"



#line 15 "mgmt_parser.c"
static const int mgmt_parser_start = 1;
static const int mgmt_parser_first_final = 26;
static const int mgmt_parser_error = 0;

static const int mgmt_parser_en_invalid_line = 23;
static const int mgmt_parser_en_main = 1;


#line 52 "mgmt_parser.rl"

void mog_mgmt_reset_parser(struct mog_mgmt *mgmt)
{
	int cs;
	
#line 30 "mgmt_parser.c"
	{
	cs = mgmt_parser_start;
	}

#line 57 "mgmt_parser.rl"
	mgmt->cs = cs;
	mgmt->mark[0] = mgmt->mark[1] = 0;
}

void mog_mgmt_init(struct mog_mgmt *mgmt, struct mog_svc *svc)
{
	memset(mgmt, 0, sizeof(struct mog_mgmt));
	mog_mgmt_reset_parser(mgmt);
	mgmt->svc = svc;
}

enum mog_parser_state
mog_mgmt_parse(struct mog_mgmt *mgmt, char *buf, size_t len)
{
	char *p, *pe, *eof = NULL;
	int cs = mgmt->cs;
	int really_done = 0;
	size_t off = mgmt->offset;

	assert(mgmt->wbuf == NULL && "unwritten data in buffer");
	assert(off <= len && "mgmt offset past end of buffer");

	p = buf + off;
	pe = buf + len;

	assert((void *)(pe - p) == (void *)(len - off) &&
	       "pointers aren't same distance");

	
#line 65 "mgmt_parser.c"
	{
	if ( p == pe )
		goto _test_eof;
	switch ( cs )
	{
case 1:
	switch( (*p) ) {
		case 9: goto st2;
		case 10: goto tr2;
		case 13: goto st3;
		case 32: goto st2;
		case 77: goto st4;
		case 115: goto st10;
		case 119: goto st17;
	}
	goto tr0;
tr0:
#line 44 "mgmt_parser.rl"
	{
		p = buf;
		p--;
		{goto st23;}
	}
	goto st0;
#line 90 "mgmt_parser.c"
st0:
cs = 0;
	goto _out;
st2:
	if ( ++p == pe )
		goto _test_eof2;
case 2:
	switch( (*p) ) {
		case 9: goto st2;
		case 10: goto tr2;
		case 13: goto st3;
		case 32: goto st2;
	}
	goto tr0;
tr2:
#line 41 "mgmt_parser.rl"
	{ mog_mgmt_fn_blank(mgmt); {p++; cs = 26; goto _out;} }
	goto st26;
tr11:
#line 29 "mgmt_parser.rl"
	{ mgmt->mark[1] = p - buf; }
#line 30 "mgmt_parser.rl"
	{ mog_mgmt_fn_md5(mgmt, buf, len); {p++; cs = 26; goto _out;} }
	goto st26;
tr14:
#line 30 "mgmt_parser.rl"
	{ mog_mgmt_fn_md5(mgmt, buf, len); {p++; cs = 26; goto _out;} }
	goto st26;
tr20:
#line 24 "mgmt_parser.rl"
	{ mgmt->mark[1] = p - buf; }
#line 25 "mgmt_parser.rl"
	{ mog_mgmt_fn_size(mgmt, buf); {p++; cs = 26; goto _out;} }
	goto st26;
tr23:
#line 25 "mgmt_parser.rl"
	{ mog_mgmt_fn_size(mgmt, buf); {p++; cs = 26; goto _out;} }
	goto st26;
tr28:
#line 32 "mgmt_parser.rl"
	{
		static int have_iostat = 1;

		if (have_iostat)
			mgmt->forward = MOG_IOSTAT;
		else
			mog_mgmt_fn_watch_err(mgmt);
		{p++; cs = 26; goto _out;}
	}
	goto st26;
st26:
	if ( ++p == pe )
		goto _test_eof26;
case 26:
#line 145 "mgmt_parser.c"
	goto tr0;
st3:
	if ( ++p == pe )
		goto _test_eof3;
case 3:
	if ( (*p) == 10 )
		goto tr2;
	goto tr0;
st4:
	if ( ++p == pe )
		goto _test_eof4;
case 4:
	if ( (*p) == 68 )
		goto st5;
	goto tr0;
st5:
	if ( ++p == pe )
		goto _test_eof5;
case 5:
	if ( (*p) == 53 )
		goto st6;
	goto tr0;
st6:
	if ( ++p == pe )
		goto _test_eof6;
case 6:
	if ( (*p) == 32 )
		goto st7;
	goto tr0;
st7:
	if ( ++p == pe )
		goto _test_eof7;
case 7:
	if ( (*p) == 47 )
		goto tr10;
	goto tr0;
tr10:
#line 28 "mgmt_parser.rl"
	{ mgmt->mark[0] = p - buf; }
	goto st8;
st8:
	if ( ++p == pe )
		goto _test_eof8;
case 8:
#line 190 "mgmt_parser.c"
	switch( (*p) ) {
		case 10: goto tr11;
		case 13: goto tr12;
	}
	if ( (*p) < 65 ) {
		if ( 45 <= (*p) && (*p) <= 57 )
			goto st8;
	} else if ( (*p) > 90 ) {
		if ( 97 <= (*p) && (*p) <= 122 )
			goto st8;
	} else
		goto st8;
	goto tr0;
tr12:
#line 29 "mgmt_parser.rl"
	{ mgmt->mark[1] = p - buf; }
	goto st9;
st9:
	if ( ++p == pe )
		goto _test_eof9;
case 9:
#line 212 "mgmt_parser.c"
	if ( (*p) == 10 )
		goto tr14;
	goto tr0;
st10:
	if ( ++p == pe )
		goto _test_eof10;
case 10:
	if ( (*p) == 105 )
		goto st11;
	goto tr0;
st11:
	if ( ++p == pe )
		goto _test_eof11;
case 11:
	if ( (*p) == 122 )
		goto st12;
	goto tr0;
st12:
	if ( ++p == pe )
		goto _test_eof12;
case 12:
	if ( (*p) == 101 )
		goto st13;
	goto tr0;
st13:
	if ( ++p == pe )
		goto _test_eof13;
case 13:
	if ( (*p) == 32 )
		goto st14;
	goto tr0;
st14:
	if ( ++p == pe )
		goto _test_eof14;
case 14:
	if ( (*p) == 47 )
		goto tr19;
	goto tr0;
tr19:
#line 23 "mgmt_parser.rl"
	{ mgmt->mark[0] = p - buf; }
	goto st15;
st15:
	if ( ++p == pe )
		goto _test_eof15;
case 15:
#line 259 "mgmt_parser.c"
	switch( (*p) ) {
		case 10: goto tr20;
		case 13: goto tr21;
	}
	if ( (*p) < 65 ) {
		if ( 45 <= (*p) && (*p) <= 57 )
			goto st15;
	} else if ( (*p) > 90 ) {
		if ( 97 <= (*p) && (*p) <= 122 )
			goto st15;
	} else
		goto st15;
	goto tr0;
tr21:
#line 24 "mgmt_parser.rl"
	{ mgmt->mark[1] = p - buf; }
	goto st16;
st16:
	if ( ++p == pe )
		goto _test_eof16;
case 16:
#line 281 "mgmt_parser.c"
	if ( (*p) == 10 )
		goto tr23;
	goto tr0;
st17:
	if ( ++p == pe )
		goto _test_eof17;
case 17:
	if ( (*p) == 97 )
		goto st18;
	goto tr0;
st18:
	if ( ++p == pe )
		goto _test_eof18;
case 18:
	if ( (*p) == 116 )
		goto st19;
	goto tr0;
st19:
	if ( ++p == pe )
		goto _test_eof19;
case 19:
	if ( (*p) == 99 )
		goto st20;
	goto tr0;
st20:
	if ( ++p == pe )
		goto _test_eof20;
case 20:
	if ( (*p) == 104 )
		goto st21;
	goto tr0;
st21:
	if ( ++p == pe )
		goto _test_eof21;
case 21:
	switch( (*p) ) {
		case 10: goto tr28;
		case 13: goto st22;
	}
	goto tr0;
st22:
	if ( ++p == pe )
		goto _test_eof22;
case 22:
	if ( (*p) == 10 )
		goto tr28;
	goto tr0;
st23:
	if ( ++p == pe )
		goto _test_eof23;
case 23:
	switch( (*p) ) {
		case 9: goto st23;
		case 13: goto st0;
		case 32: goto st23;
	}
	goto tr30;
tr30:
#line 14 "mgmt_parser.rl"
	{ mgmt->mark[0] = p - buf; }
	goto st24;
tr33:
#line 15 "mgmt_parser.rl"
	{ mgmt->mark[1] = p - buf; }
	goto st24;
st24:
	if ( ++p == pe )
		goto _test_eof24;
case 24:
#line 351 "mgmt_parser.c"
	switch( (*p) ) {
		case 9: goto tr34;
		case 10: goto tr35;
		case 13: goto tr34;
		case 32: goto tr34;
	}
	goto tr33;
tr34:
#line 15 "mgmt_parser.rl"
	{ mgmt->mark[1] = p - buf; }
	goto st25;
st25:
	if ( ++p == pe )
		goto _test_eof25;
case 25:
#line 367 "mgmt_parser.c"
	if ( (*p) == 10 )
		goto tr37;
	goto st25;
tr37:
#line 17 "mgmt_parser.rl"
	{
		mog_mgmt_fn_unknown(mgmt, buf);
		really_done = 1;
		{p++; cs = 27; goto _out;}
	}
	goto st27;
st27:
	if ( ++p == pe )
		goto _test_eof27;
case 27:
#line 383 "mgmt_parser.c"
	goto st0;
tr35:
#line 15 "mgmt_parser.rl"
	{ mgmt->mark[1] = p - buf; }
#line 17 "mgmt_parser.rl"
	{
		mog_mgmt_fn_unknown(mgmt, buf);
		really_done = 1;
		{p++; cs = 28; goto _out;}
	}
	goto st28;
st28:
	if ( ++p == pe )
		goto _test_eof28;
case 28:
#line 399 "mgmt_parser.c"
	switch( (*p) ) {
		case 9: goto tr34;
		case 10: goto tr35;
		case 13: goto tr34;
		case 32: goto tr34;
	}
	goto tr33;
	}
	_test_eof2: cs = 2; goto _test_eof; 
	_test_eof26: cs = 26; goto _test_eof; 
	_test_eof3: cs = 3; goto _test_eof; 
	_test_eof4: cs = 4; goto _test_eof; 
	_test_eof5: cs = 5; goto _test_eof; 
	_test_eof6: cs = 6; goto _test_eof; 
	_test_eof7: cs = 7; goto _test_eof; 
	_test_eof8: cs = 8; goto _test_eof; 
	_test_eof9: cs = 9; goto _test_eof; 
	_test_eof10: cs = 10; goto _test_eof; 
	_test_eof11: cs = 11; goto _test_eof; 
	_test_eof12: cs = 12; goto _test_eof; 
	_test_eof13: cs = 13; goto _test_eof; 
	_test_eof14: cs = 14; goto _test_eof; 
	_test_eof15: cs = 15; goto _test_eof; 
	_test_eof16: cs = 16; goto _test_eof; 
	_test_eof17: cs = 17; goto _test_eof; 
	_test_eof18: cs = 18; goto _test_eof; 
	_test_eof19: cs = 19; goto _test_eof; 
	_test_eof20: cs = 20; goto _test_eof; 
	_test_eof21: cs = 21; goto _test_eof; 
	_test_eof22: cs = 22; goto _test_eof; 
	_test_eof23: cs = 23; goto _test_eof; 
	_test_eof24: cs = 24; goto _test_eof; 
	_test_eof25: cs = 25; goto _test_eof; 
	_test_eof27: cs = 27; goto _test_eof; 
	_test_eof28: cs = 28; goto _test_eof; 

	_test_eof: {}
	if ( p == eof )
	{
	switch ( cs ) {
	case 1: 
	case 2: 
	case 3: 
	case 4: 
	case 5: 
	case 6: 
	case 7: 
	case 8: 
	case 9: 
	case 10: 
	case 11: 
	case 12: 
	case 13: 
	case 14: 
	case 15: 
	case 16: 
	case 17: 
	case 18: 
	case 19: 
	case 20: 
	case 21: 
	case 22: 
#line 44 "mgmt_parser.rl"
	{
		p = buf;
		p--;
		{goto st23;}
	}
	break;
#line 469 "mgmt_parser.c"
	}
	}

	_out: {}
	}

#line 86 "mgmt_parser.rl"

	if (really_done)
		cs = mgmt_parser_first_final;

	mgmt->cs = cs;
	mgmt->offset = p - buf;

	if (cs == mgmt_parser_error)
		return MOG_PARSER_ERROR;

	assert(p <= pe && "buffer overflow after mgmt parse");
	assert(mgmt->offset <= len && "offset longer than len");

	if (mgmt->cs == mgmt_parser_first_final) return MOG_PARSER_DONE;
	return MOG_PARSER_CONTINUE;
}
